// nLife Editor
//  An editor plugin for nLife
//  http://www.node99.net/projects/nLife

// COPYRIGHT
// Copyright (c) 2009, Kyle Hubert, Douglas Haber
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above 
//       copyright notice, this list of conditions and the following
//       disclaimer in the documentation and/or other materials provided
//       with the distribution.
//     * The names names of the authors may not be used to endorse or
//       promote products derived from this software without specific
//       prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND ITS
// CONTRIBUTERS ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR ITS
// CONTRIBUTERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
// USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

// TODO: 
//  * Add a bottom bar and get rid of the doubleClick stuff
//  * Regex parse all inputs
//  * Add keyboard support
//  * Hex values are wrong in numToHex()

function numToHex(n) {
  //TODO:  This doesn't work right.  Fix it.
  var h = "0123456789ABCDEF";
  var b = 16;
  var str = '';
  
  var em = 10;
  
  while ( n > 0) {
    r = n % b;
    n = n - (r * b);
    str += h.charAt(r);

    if(--em == 0) { break; }
  }
  
  while(str.length < 2) {
    str = '0' + str;
  }
  
  return(str);
}
function colorToHex(str) {
  // Convert a color string in decimal notation to hex
  var c = str.match( /^rgb\((\d+), (\d+), (\d+)\)$/ );
  if(c.length == 4) {
    colorstr = numToHex(c[1]) + numToHex(c[2]) + numToHex(c[3]);
    return('#' + colorstr);
  }
  return(str);
}
  
function setDeadCellColorStr() {
  // Set the dead cell color based on the given user string
  //TODO regex check the input
  if(life.editor.controls.deadCellColor.value) {
    life.deadCellColor = life.editor.controls.deadCellColor.value;
    redrawBoard();
  }
}

function setLiveCellColorStr() {
  // Set the dead cell color based on the given user string
  //TODO regex check the input
  if(life.editor.controls.liveCellColor.value) {
    life.liveCellColor = life.editor.controls.liveCellColor.value;
    redrawBoard();
  }
}

function setDeadCellColor(div) {
  // Set the dead cell color based on the background color of the given cell
  life.deadCellColor = div.style.backgroundColor;
  life.editor.controls.deadCellColor.value = colorToHex(div.style.backgroundColor);
  redrawBoard();
}
  
function setLiveCellColor(div) {
  // Set the live cell color based on the background color of the given cell
  life.liveCellColor = div.style.backgroundColor;
  life.editor.controls.liveCellColor.value = colorToHex(div.style.backgroundColor);
  redrawBoard();
}

function getRadioValue(radio) {
  var x;
  for(x = 0 ; x < radio.length; x++) {
    if(radio[x].checked) {
      return(radio[x].value);
    }
  }
  return(null);
}

function setRadioValue(radio, val) {
  var x;
  for(x = 0 ; x < radio.length ; x++ ) {
    if(radio[x].value == val) {
      radio[x].checked = true;
      return;
    }
  }
}

function gridChangeHandler() {
  var x, y;
  
  if(getRadioValue(life.editor.controls.gridRadio) == 1) {
    for(x = 0 ; x < life.boardHeight ; x++ )
      for(y = 0 ; y < life.boardWidth ; y++ )
        life.board[x][y].style.borderStyle = 'dotted';
  }
  else {
    for(x = 0 ; x < life.boardHeight ; x++ )
      for(y = 0 ; y < life.boardWidth ; y++ )
        life.board[x][y].style.borderStyle = 'none';
  }
}  

function updateRateChangeHandler() {
  if(life.editor.controls.updateRate.value <= 0 ||
     isNaN(parseInt(life.editor.controls.updateRate.value)) ) {
    life.editor.controls.updateRate.value = 1;
  } // beyond about 30 there isn't much difference, so cap it there.
  else if(life.editor.controls.updateRate.value > 30) {
    life.editor.controls.updateRate.value = 30;
  }
  life.updateTime = 1000 / life.editor.controls.updateRate.value;
}

function onPause() {
  life.editor.controls.pause.disabled = true;
  life.editor.controls.play.disabled = false;
  pause();
}
  
function onPlay() {
  life.editor.controls.play.disabled = true;
  life.editor.controls.pause.disabled = false;
  play();
}

function onStoreBoard() {
  life.editor.controls.restoreButton.disabled = '';
  storeBoard();
}
 
function controlsHTML() {
  // TODO:  Don't hardcode the GUI in a string.
  return(
         "<button id='pauseButton' type='button' onclick='onPause()'>Pause</button>"+
         "<button type='button' id='playButton' onclick='onPlay()' disabled='disabled'>Play</button>"+
         "<button type='button' onclick='clearBoard()'>Clear</button><br>"+
         "<button type='button' onclick='onStoreBoard()'>Store</button>"+
         "<button type='button' onclick='restoreBoard()' "+
         "        id='restoreButton' disabled='disabled'>Restore</button>"+
         "<hr>Updates Per Second: <input id='updateRate' type='text' "+
         "   size='5' maxlength='5' onChange='updateRateChangeHandler()'>"+
         "<hr><b>Dead Cell Color</b>"+
         "<div class='colorPicker'>"+
         "<div style='background-color:#ff0000' onClick='setDeadCellColor(this)'></div>"+
         "<div style='background-color:#ffa800' onClick='setDeadCellColor(this)'></div>"+
         "<div style='background-color:#ffff00' onClick='setDeadCellColor(this)'></div>"+
         "<div style='background-color:#18ff00' onClick='setDeadCellColor(this)'></div>"+
         "<div style='background-color:#0084ff' onClick='setDeadCellColor(this)'></div>"+
         "<div style='background-color:#ae00ff' onClick='setDeadCellColor(this)'></div>"+
         "<div style='background-color:#f600ff' onClick='setDeadCellColor(this)'></div>"+
         "<div style='background-color:#ffffff' onClick='setDeadCellColor(this)'></div>"+
         "<div style='background-color:#9b9b9b' onClick='setDeadCellColor(this)'></div>"+
         "<div style='background-color:#000000' onClick='setDeadCellColor(this)'></div>"+
         "Color: <input id='deadCellColorInput' type='text' size='7' maxlength='7'>"+
         " <button type='button' onclick='setDeadCellColorStr()'>Set</button>"+
         "</div><br>"+
         "<hr><b>Live Cell Color</b>"+
         "<div class='colorPicker'>"+
         "<div style='background-color:#ff0000' onClick='setLiveCellColor(this)'></div>"+
         "<div style='background-color:#ffa800' onClick='setLiveCellColor(this)'></div>"+
         "<div style='background-color:#ffff00' onClick='setLiveCellColor(this)'></div>"+
         "<div style='background-color:#18ff00' onClick='setLiveCellColor(this)'></div>"+
         "<div style='background-color:#0084ff' onClick='setLiveCellColor(this)'></div>"+
         "<div style='background-color:#ae00ff' onClick='setLiveCellColor(this)'></div>"+
         "<div style='background-color:#f600ff' onClick='setLiveCellColor(this)'></div>"+
         "<div style='background-color:#ffffff' onClick='setLiveCellColor(this)'></div>"+
         "<div style='background-color:#9b9b9b' onClick='setLiveCellColor(this)'></div>"+
         "<div style='background-color:#000000' onClick='setLiveCellColor(this)'></div>"+
         "Color: <input id='liveCellColorInput' type='text' size='7' maxlength='7'>"+
         " <button type='button' onclick='setLiveCellColorStr()'>Set</button>"+
         "</div><br>"+
         "<hr><b>Grid</b>&nbsp;&nbsp; "+
         "<input onChange='gridChangeHandler()' name='grid' type='radio' value='1'>On"+
         "<input onChange='gridChangeHandler()' name='grid' type='radio' value='0'>Off<br>"
         );
}

function onMouseDown(e) {
  if (!e) { e = window.event };
  if(e.preventDefault && e.target.name == 'cell') { e.preventDefault(); }
  life.mouseDown = 1;
}

function onMouseUp() {
  life.mouseDown = 0;
  if(life.stopped) {
    setTimeout('doCycle()', 500);
    life.stopped = 0;
  }
}

function editorDoCycle() {
  if(life.mouseDown) { return(0); }
  else               { return(1); }
}

function toggleControls() {
  if(life.editor.controlsDiv.style.visibility == 'visible') {
    life.editor.controlsDiv.style.visibility = 'hidden';
  }
  else {
    life.editor.controlsDiv.style.visibility = 'visible';
  }
}

function setupControls() {
  life.editor.controls = {
    pause         : document.getElementById('pauseButton'),
    play          : document.getElementById('playButton'),
    deadCellColor : document.getElementById('deadCellColorInput'),
    liveCellColor : document.getElementById('liveCellColorInput'),
    restoreButton : document.getElementById('restoreButton'),
    updateRate    : document.getElementById('updateRate'),
    gridRadio     : document.getElementsByName('grid')
  };
  setRadioValue(life.editor.controls.gridRadio, '1');
  life.editor.controls.updateRate.value = 1000 / life.updateTime;
}

function initControls() {
  life.editor.controlsDiv = document.createElement('div')
  life.editor.controlsDiv.className = 'controls';
  life.editor.controlsDiv.innerHTML = controlsHTML();

  // NOTE:  When in the style sheet we couldn't read the height/width back,
  //        so these next to lines are in the code.
  life.editor.controlsDiv.style.height = '350px';
  life.editor.controlsDiv.style.width = '220px';
  life.editor.controlsDiv.style.top  = (document.body.clientHeight -
                                  parseInt(life.editor.controlsDiv.style.height) + 10) / 2;
  life.editor.controlsDiv.style.left = document.body.clientWidth -
    parseInt(life.editor.controlsDiv.style.width) - 20;

  setTimeout("setupControls()", 100);
  document.body.appendChild(life.editor.controlsDiv);
  document.ondblclick = toggleControls;
}

function onIntroMessageClose() {
  play();
  clearMessage();
}

function clearMessage() {
  var div = document.getElementById("messageDiv");
  div.parentNode.removeChild(div);
}

function message(msg, height, width) {
  // Create a message popup.  This only supports 1 message at a time.
  // TODO: if height or width is greater than the window size, max it out
  var div = document.createElement("div");
  div.innerHTML = msg;
  div.id = "messageDiv";
  div.style.height = height + 'px';
  div.style.width = width + 'px';
  div.style.position = 'absolute';

  // TODO:  Make sure this works in other browsers than FF/Chrome
  div.style.top = (window.innerHeight - height) / 2 + 'px';
  div.style.left = (window.innerWidth - width) / 2 + 'px';

  document.body.appendChild(div);
}

function initIntroMessage() {
  message("<h2>Welcome to nLife</h2>"+
	  "<h3>Copyright (c) 2009, Kyle Hubert, Douglas Haber<br>All rights reserved.</h3>"+
	  "<ul><li>To bring up the controls double click on the window.</li>"+
	  "<li>To draw, hold the left mouse button down and move the mouse.</li></ul><br>"+
	  "<center>This works in Firefox and Google Chrome.  For best performance, use Chrome. "+
	  "Internet Explorer is untested.<br><br>"+
	  "For more information on this software or to download a copy "+
	  "click <a href=\"http://www.node99.net/projects/nLife\">here.</a><br>"+
	  "<br><br><button onclick=\"onIntroMessageClose()\">"+
	  "Start</button></center><div id=\"version\">v"+life.version+"</div>",
	  400, 600);
  pause();
}

function initEditor() {
  life.editor = {
    lastDraw : 0,  // Last time the user did a drawing
    mouseDown : 0
  };
  
  document.onmousedown = onMouseDown;
  document.onmouseup = onMouseUp;

  initControls();
  initIntroMessage();
}

registerInit( initEditor );
registerCycle( editorDoCycle );
