//  nLife
//     node99Life ? 
//     noLife ?
//     Notorious Living Idealistic Flickering Emulator ?
//  A quick and dirty Conway life like program
//  http://www.node99.net/projects/nLife

// COPYRIGHT
// Copyright (c) 2009, Kyle Hubert, Douglas Haber
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above 
//       copyright notice, this list of conditions and the following
//       disclaimer in the documentation and/or other materials provided
//       with the distribution.
//     * The names names of the authors may not be used to endorse or
//       promote products derived from this software without specific
//       prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND ITS
// CONTRIBUTERS ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR ITS
// CONTRIBUTERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
// USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

//  PLUGINS
//  A very simple plugin interface exists.  A plugin should
//  be loaded in source after nlife.js.   It can then use the
//  functions of registerInit() and registerCycle() to register
//  functions to be run at init and every cycle.  The cycle
//  function can return a false value to stop the cycle process
//  and place the program into a stopped state.

//  TODO 
//  * Switch everything to a canvas
//  * Fix the sizing problem, make scaling work
//  * Test in browsers other than Chrome and FF
//  * Allow for dynamic changes to the scaling
//  * SEE ALSO:  The TODO list in editor.js

var life = {
  version        : '0.5',
  updateTime     : 200,
  chanceOfLife   : 8,
  scale          : 30,
  boardDiv       : null,  // Set during init
  initFunctions  : new Array, 
  cycleFunctions : new Array, 
  board          : new Array, // The grid
  tmpBoard       : new Array, // Temporary grid for simultaneous stepping
  storeBoard     : new Array, // A temporary board for storing a state
  boardStored    : 0, 
  liveCellColor  : '#000000',
  deadCellColor  : '#9b9b9b',
  paused         : 0,
  stopped        : 0  // If the doCycle() updates have ceased
};


function isAlive(x, y) {
    if(x < 0 || 
       y < 0 || 
       x >= life.boardHeight ||
       y >= life.boardWidth) { 
      return(0); 
    }
    else if(life.board[x][y].alive == 1) { return(1); }
    else { return(0); }

}

function countNeighbors(posx, posy) {
  return(isAlive(posx - 1, posy - 1) +
	 isAlive(posx - 1, posy) +
	 isAlive(posx - 1, posy + 1) +
	 isAlive(posx, posy - 1) +
	 isAlive(posx, posy + 1) +
	 isAlive(posx + 1, posy - 1) +
	 isAlive(posx + 1, posy) +
	 isAlive(posx + 1, posy + 1)
	 );
}

function testLife(x,y) {
    var n = countNeighbors(x, y);
    if(isAlive(x, y)) {
	if( (n < 2) || (n > 3) ) {
	    return 0;
	}
	return 1;
    } else if (n == 3) {
	return 1;
    }
    return 0;
}

function updateBoard() {
  var x, y;

  for(x = 0 ; x < life.boardHeight ; x++ ) {
      for(y = 0 ; y < life.boardWidth ; y++ ) {
	if(life.tmpBoard[x][y]) {
	  life.board[x][y].style.backgroundColor = life.liveCellColor;
	  life.board[x][y].alive = 1;
	}
	else {
          life.board[x][y].style.backgroundColor = life.deadCellColor;
          life.board[x][y].alive = 0;
        }
      }
  }    
}

function redrawBoard() {
  var x, y;
  
  for(x = 0 ; x < life.boardHeight ; x++ ) {
    for(y = 0 ; y < life.boardWidth ; y++ ) {
      if(life.board[x][y].alive) {
	life.board[x][y].style.backgroundColor = life.liveCellColor;
      }
      else {
	life.board[x][y].style.backgroundColor = life.deadCellColor;
      }
    }
  }
}

function restoreBoard() {
  if(!life.boardStored) { return; }

  for(x = 0 ; x < life.boardHeight ; x++ ) {
    for(y = 0 ; y < life.boardWidth ; y++ ) {
      life.board[x][y].alive = life.storeBoard[x][y];
    }
  }

  redrawBoard();
}

function storeBoard() {
  var x, y;
  
  if(!life.boardStored) {
    life.boardStored = 1;
  }

  for(x = 0 ; x < life.boardHeight ; x++ ) {
    for(y = 0 ; y < life.boardWidth ; y++ ) {
      life.storeBoard[x][y] = life.board[x][y].alive;
    }
  }
}

function clearBoard() {
  var x, y;
  
  for(x = 0 ; x < life.boardHeight ; x++ ) {
    for(y = 0 ; y < life.boardWidth ; y++ ) {
      life.board[x][y].alive = 0; 
    }
  }
  redrawBoard();
}

function doCycle() {
  var x, y;
  var d = new Date;

  // Run each registered cycle function.   
  // If they return 0 go to a stopped state.
  for(x = 0 ; x < life.cycleFunctions.length ; x++) {
    if(! life.cycleFunctions[x]() ) { 
      life.stopped = 1;
      return; 
    }  
  }

  if(life.paused || life.stopped || life.mouseDown) {
    life.stopped = 1;
    return; 
  }
  
  for(x = 0 ; x < life.boardHeight ; x++ ) {
    for(y = 0 ; y < life.boardWidth ; y++ ) {
      life.tmpBoard[x][y] = testLife(x,y);
    }
  }

  updateBoard();
  setTimeout('doCycle()', life.updateTime);
}

function toggleAlive(e) {
  var d = new Date;
  if (!e) { // IE
    e = window.event;
    e.target = e.srcElement; 
  }  

  if(e.target.alive) {
    e.target.style.backgroundColor = life.deadCellColor;
    e.target.alive = 0;
  }
  else {
    e.target.style.backgroundColor = life.liveCellColor;
    e.target.alive = 1;
  }
  
  life.lastDraw = d.getTime();
}

function setAlive(e) {
  if(!life.mouseDown) { return; }
  var d = new Date;
  if (!e) { // IE
    e = window.event;
    e.target = e.srcElement; 
  }  
  e.target.style.backgroundColor = life.liveCellColor;
  e.target.alive = 1;
  life.lastDraw = d.getTime();
}

function createCell(x, y, alive) { 
  var cell = document.createElement("div");
  cell.name = 'cell';
  cell.className = 'cell';
  cell.style.top = x * life.scale;
  cell.style.left = y * life.scale;
  cell.style.height = life.scale;
  cell.style.width = life.scale;

  if(alive) {
    cell.style.backgroundColor = life.liveCellColor;
    cell.alive = 1;
  }
  else {
    cell.style.backgroundColor = life.deadCellColor;
    cell.alive = 0;
  }
  cell.onmouseover = setAlive;
  cell.onclick = toggleAlive;

  return(cell);
}

function pause() { life.paused = 1; }

function play() {
  life.paused = 0;
  if(life.stopped) {
    setTimeout('doCycle()', 100);
    life.stopped = 0;
  }
}

function initBoard() {
  var x, y;

  life.boardHeight = parseInt(life.boardDiv.style.height) / life.scale;
  life.boardWidth  = parseInt(life.boardDiv.style.width)  / life.scale;
  
  for(x = 0 ; x < life.boardHeight ; x++ ) {
    life.board[x]    = new Array;
    life.tmpBoard[x] = new Array;
    life.storeBoard[x] = new Array;
    for(y = 0 ; y < life.boardWidth ; y++ ) {
      life.board[x][y] = createCell(x, y, 
		(Math.floor(Math.random() * life.chanceOfLife) == 0) );
      life.boardDiv.appendChild(life.board[x][y]);
    }
  }
}

function registerInit(func)  { life.initFunctions.push(func);  }
function registerCycle(func) { life.cycleFunctions.push(func); } 

function init() {
  var e;

  life.boardDiv = document.getElementById("nLifeBoard"),
  life.boardDiv.style.height = document.body.clientHeight - life.scale;
  life.boardDiv.style.width  = document.body.clientWidth - life.scale;

  // Run all init functions
  for(x = 0 ; x < life.initFunctions.length ; x++) {
    life.initFunctions[x]();
  }

  if(life.boardDiv) {
      initBoard();
      setTimeout('doCycle()', life.updateTime);
  } 
  else {
    alert("ERROR: Could not find the Board Div.  Aborting!\n");
  }
}

window.onload = init;
